#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <TROOT.h>
#include <iostream>
#include <string>
#include "LHAPDF/LHAPDF.h"
#include <iterator>
#include <iostream>
#include <cstdlib>

using namespace std;
using namespace LHAPDF;

/*
	Parton evolution:
      1. generate starting distribution in x and add intrisnic transverse mom.
      2. evolve sarting distribution to large values of t
      
      Plot starting distribution and intrinsic kt
      Plot evolved distribution as function of x and kt
      
	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
	http://luscher.web.cern.ch/luscher/ranlux/index.html      

*/


double gauss ( double mean, double sigma )
{	
	int n1, npoints = 12;
	double rvec[12];
      double result, r;
      result = 0.;
      r = 0;
      ranlxd(rvec,12);
	for ( n1 = 0; n1 < npoints; n1++ ) {
		r = r + rvec[n1] ;
	}
      result =mean + sigma * (r - 6.);
      return(abs(result));
}

void get_starting_pdf(double xmin, double q2, double& weightx, double& x, double& kx, double& ky, double& kt2)
{
 		static const double pi = 3.14159265358979323848;
		double xmax = 1;
            double pdf;
            double phi;
		double rvec[2];
      	ranlxd(rvec,2);
// get x value according to g(x)\sim 1/x            
	 	x = xmin*pow(xmax/xmin,rvec[0]);
// use gluon: xg(x) = 3 (1-x)**5
            pdf = pow((1-x),5.)*3./x ;
      	weightx = x*log(xmax/xmin) * pdf ;
// now generate instrinsic kt according to a gauss distribution            
      	kt2 = gauss(0,0.7);
      	phi = 2.*pi*rvec[1];            
            kx = sqrt(kt2)*cos(phi);
            ky = sqrt(kt2)*sin(phi);
//            cout << "in getpdf:  x " << x << " kx " << kx <<" ky " << ky <<endl; 
}

void sudakov(double t0, double& t)
{
//		here we calculate  from the sudakov form factor the next t > t0
 		static const double pi = 3.14159265358979323848;
		double epsilon=0.1;
            double as2pi=0.1/2./pi,Cf=4./3.;
// for Pqq use fact = Cf
            double fac=Cf; 
            double r1,t2;
		double rvec[1];
      	ranlxd(rvec,1);
//		frist do the simple case with fixed alphas and only the 1/(1-z) term of the splitting fct
		r1 = rvec[0];
            t2 = log(r1)/fac/as2pi/log(epsilon);
            t2 = t0 * exp(t2);
            t = t2;   
//            cout << " t0= "<<t0<< " t= "<<t2<< " epsilon= "<<epsilon <<endl;
            if ( t2 < t0 ) {        
            	cout << "FATAL t0 > t:  t0= "<<t0<< "t= "<<t2<< "epsilon= "<<epsilon <<endl;
            }
}

void splitting(double& z, double& weightz)
{
		double epsilon=0.1;
 		static const double pi = 3.14159265358979323848;
            double as2pi=0.1/2./pi;
            double r1;
		double rvec[2];
      	ranlxd(rvec,2);
//		here we calculate  the splitting variable z
//
//		approximate Pqq(z) with 1/(1-z) and generate z according to this
//		calculate according to Pqq = 4/3*as2p*(1+z**2)/(1-z)
		r1 = rvec[0];
		z = 1. - pow(epsilon,r1);
// 		the weight is only the ratio of the (1+z**2)/(1-z)  and 1/(1-z) terms
            weightz = (1.+z*z);
//
               
//            cout << "splitting z= "<<z << " weightz = " << weightz <<endl;
		
}

void evolve_pdf(double xmin, double q2, double x0, double kx0, double ky0, double kt20,  double& weightx, double& x, double& kx, double& ky, double& kt2)
{
 		static const double pi = 3.14159265358979323848;
		double t0=1.,t1,tcut,phi;
            double z=0, weightz;
		double epsilon=0.1;
            double ratio_splitting;
		double rvec[2];
            x = x0;
            kx = kx0;
            ky = ky0;
            kt2 = kt20;
            weightx = 1. ;
            t1 = t0 ;
            tcut = q2;
            while (t1 < tcut ) {
      		ranlxd(rvec,2);
// 			here we do now the evolution
//			first we generate t
            	t0 = t1;
			sudakov(t0, t1) ;
//			then we generate z
			splitting(z, weightz);
// since the sudakov involves only the 1/(1-z) part of the splitting fct, we need to weight each branching
// by the ratio of the intgral of the full and approximate splitting fct
//			using Pqq
 			ratio_splitting= (-2.*log(epsilon) - 0.5*epsilon*epsilon +3./2.*epsilon - 3./2.)/(-log(epsilon));
                 
//                  cout << " t1 = " << t1 << " t0 = "<<t0<<endl;
//                  cout << " x = " << x << " z = "<<z<<endl;
//                  cout << " splitting ratio " << ratio_splitting <<endl;
                  if ( t1 < tcut ) { 
                     x = x*z;
                     weightx = weightx *weightz*ratio_splitting;
      	         phi = 2*pi*rvec[0];  
//			                             
// 			   use here the angular ordering condition: sqrt(t1) = qt/(1-z) 
//			   and apply this also to the propagator gluons                                
                      kx = kx + sqrt(t1)*cos(phi)*(1.-z);
                      ky = ky + sqrt(t1)*sin(phi)*(1.-z);                     
                      kt2 = kx*kx + ky*ky ;
//                    cout << " kt = "<<sqrt(kt2) << " phi = " << phi << " t1 = " << sqrt(t1) << endl;
//                    cout << "  t= "<< t<< " x0 = " <<x0 << "x =" << x << " z =" << z <<endl;
                     }
            }
//            cout << " evolve end  t= "<< t<< " q2 " <<q2 << " x0 = " <<x0 << "x =" << x <<endl;
}



int main(int argc,char **argv)
{
      TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);
	int n1, npoints = 1000000;

      
      double xmin = 0.00001, q2=100*100;
      double x , kx, ky, kt2, weightx ;
      double x0, kx0, ky0, kt20, weightx0 ;
      
      double a,b, binwidth1,binwidth2;
	double momsum0=0., momsum=0.,fact_mom_sumrule;
    
// initialise random number generator
//    rlxd_init( luxory level, seed )
      rlxd_init(2,32767);
      
//               TH1F("label","title",nr of bins, xlow,xhigh )
      TH1F *histo1 = new TH1F("x0","x0",100, -5, 0.);
      TH1F *histo10 = new TH1F(*histo1); 
      binwidth1=5./100.;
      TH1F *histo2 = new TH1F("kt0 ","kt0 ",100, 0, 10.);
      TH1F *histo20 = new TH1F(*histo2); 
      binwidth2=1000./1000;
      TH1F *histo3 = new TH1F("x","x",100, -5, 0.);
      TH1F *histo30 = new TH1F(*histo3); 
      TH1F *histo4 = new TH1F("kt ","kt ",1000, 0, 1000.);
      TH1F *histo40 = new TH1F(*histo4); 
            

	for ( n1 = 0; n1 < npoints; n1++ ) {

//		generate starting distribution in x and kt
      	get_starting_pdf(xmin, q2, weightx0, x0, kx0, ky0, kt20 );

//		now do the evolution	
		evolve_pdf(xmin, q2, x0, kx0, ky0, kt20, weightx, x, kx, ky, kt2);   
//            cout << " in main weightx0 " << weightx0 << " weightx " << weightx <<endl; 
            weightx = weightx0 *weightx ;         
//            cout << " in main x " << x << " kx " << kx <<" ky " << ky<<endl; 
            if ( x < 1) {
// 				weighting with 1/x0:
//				plot dxg(x)/dlogx *Jacobian, Jacobian dlogx/dx = 1/x
//                      log(x) = 2.3026 log10(x)
				momsum0 = momsum0+x0*weightx0 ;
				momsum = momsum+x*weightx ;
				histo1->Fill(log10(x0),weightx0/2.3026);
				histo2->Fill(sqrt(kt20),weightx0);
				histo3->Fill(log10(x),weightx/2.3026);
				histo4->Fill(sqrt(kt2),weightx);
		   	}
      }          
//                                  

      fact_mom_sumrule=momsum0/momsum;
//    write histogramm out to file
      TFile file("output-example-13-Pqq.root","RECREATE");
//general root settings 
      gROOT->Reset();
      gROOT->SetStyle("Plain");
      gStyle->SetPadTickY(1); // ticks at right side
      gStyle->SetOptStat(0); // get rid of statistics box
      TCanvas *c = new TCanvas("ctest", "" ,0, 0, 500, 500);
// divide the canvas in 1 parts in x and 1 in y
      c -> Divide(2,2);
      a=1./npoints/binwidth1; 
//      cout << " a " << a << "npoints" << npoints << " binwidth1 " << binwidth1<<endl;
      b=0; 
      histo10->Add(histo1,histo1,a,b); 
//      histo1->Write();
      histo10->Write();
      c -> cd(1);
//      gPad->SetLogy(0);
      histo10 -> Draw();
      a=fact_mom_sumrule/npoints/binwidth1; 
      histo30->Add(histo3,histo3,a,b); 
//      histo3->Write();
      histo30->Write();
      c -> cd(2);
//      gPad->SetLogy();
      histo30 -> Draw();
      a=1./npoints/binwidth2; 
//      cout << " a " << a << "npoints" << npoints << " binwidth2 " << binwidth2<<endl;
      histo20->Add(histo2,histo2,a,b); 
//      histo2->Write();
      histo20->Write();
      c -> cd(3);
      gPad->SetLogy();
      histo20 -> Draw();
      a=fact_mom_sumrule/npoints/binwidth1; 
      histo40->Add(histo4,histo4,a,b); 
//      histo4->Write();
      histo40->Write();
      c -> cd(4);
      gPad->SetLogy();
      histo40 -> Draw();
      c-> Draw();
      c->WaitPrimitive();
      c-> Print("example13-Pqq.pdf");
      gMyRootApp->SetReturnFromRun(true);
      file.Close();
	return(0);
}
